@@ -49,8 +49,10 @@ void rdfa_init_context(rdfacontext* context)
    context->parent_subject = NULL;
    if(context->base != NULL)
    {
+      char* cleaned_base = rdfa_iri_get_base(context->base);
       context->parent_subject =
-         rdfa_replace_string(context->parent_subject, context->base);
+         rdfa_replace_string(context->parent_subject, cleaned_base);
+      free(cleaned_base);
    }
    
    // the [parent object] is set to null;
@@ -171,6 +173,7 @@ static size_t rdfa_init_base(
             {
                size_t uri_size = uri_end - uri_start;
                char* temp_uri = (char*)malloc(sizeof(char) * uri_size + 1);
+	       char* cleaned_base;
                strncpy(temp_uri, uri_start, uri_size);
                temp_uri[uri_size] = '\0';
 
@@ -178,13 +181,15 @@ static size_t rdfa_init_base(
                //       be? Setting current_object_resource will make
                //       sure that the BASE element is inherited by all
                //       subcontexts.
+	       cleaned_base = rdfa_iri_get_base(temp_uri);
                context->current_object_resource =
-                  rdfa_replace_string(context->current_object_resource,
-                                      temp_uri);
+                  rdfa_replace_string(
+                     context->current_object_resource, cleaned_base);
+
+	       // clean up the base context
                context->base =
-                  rdfa_replace_string(context->base,
-                                      temp_uri);
-               
+                  rdfa_replace_string(context->base, cleaned_base);
+               free(cleaned_base);
                free(temp_uri);
             }
          }         
@@ -608,8 +613,16 @@ static void XMLCALL
          else if(strcmp(attr, "datatype") == 0)
          {
             datatype_curie = value;
-            datatype = rdfa_resolve_curie(context, datatype_curie,
-               CURIE_PARSE_INSTANCEOF_DATATYPE);
+
+            if(strlen(datatype_curie) == 0)
+            {
+               datatype = rdfa_replace_string(datatype, "");
+            }
+            else
+            {
+               datatype = rdfa_resolve_curie(context, datatype_curie,
+                  CURIE_PARSE_INSTANCEOF_DATATYPE);
+            }
          }
 #ifndef LIBRDFA_IN_RAPTOR
          else if(strcmp(attr, "xml:lang") == 0)
@@ -630,8 +643,11 @@ static void XMLCALL
    }
 
 #ifdef LIBRDFA_IN_RAPTOR
-   if(context->sax2)
-      xml_lang=(const char*)raptor_sax2_inscope_xml_language(context->sax2);
+   if(context->sax2) {
+      xml_lang = (const char*)raptor_sax2_inscope_xml_language(context->sax2);
+      if(!xml_lang)
+        xml_lang = "";
+   }
 #endif
    // check to see if we should append an xml:lang to the XML Literal
    // if one is defined in the context and does not exist on the element.
@@ -1053,9 +1069,12 @@ rdfacontext* rdfa_create_context(const char* base)
    // if the base isn't specified, don't create a context
    if(base_length > 0)
    {
+      char* cleaned_base;
       rval = (rdfacontext*)malloc(sizeof(rdfacontext));
       rval->base = NULL;
-      rval->base = rdfa_replace_string(rval->base, base);
+      cleaned_base = rdfa_iri_get_base(base);
+      rval->base = rdfa_replace_string(rval->base, cleaned_base);
+      free(cleaned_base);
 
       /* parse state */
       rval->wb_allocated = 0;
@@ -1275,7 +1294,11 @@ int rdfa_parse_chunk(rdfacontext* context, char* data, size_t wblen, int done)
       if(XML_Parse(context->parser, context->working_buffer,
          context->wb_offset, 0) == XML_STATUS_ERROR)
       {
+#ifdef WIN32
+         printf(
+#else
          fprintf(stderr,
+#endif
                  "%s at line %d, column %d\n",
                  XML_ErrorString(XML_GetErrorCode(context->parser)),
                  XML_GetCurrentLineNumber(context->parser),
@@ -1298,7 +1321,11 @@ int rdfa_parse_chunk(rdfacontext* context, char* data, size_t wblen, int done)
 #else
    if(XML_Parse(context->parser, data, wblen, done) == XML_STATUS_ERROR)
    {
-      fprintf(stderr,
+#ifdef WIN32
+         printf(
+#else
+         fprintf(stderr,
+#endif
               "%s at line %d, column %d.\n",
               XML_ErrorString(XML_GetErrorCode(context->parser)),
               XML_GetCurrentLineNumber(context->parser),
