@@ -5,7 +5,7 @@
  * N-Triples
  * http://www.w3.org/TR/rdf-testcases/#ntriples
  *
- * Copyright (C) 2001-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2001-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2001-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -120,14 +120,15 @@ static const char * const term_type_strings[]={
 };
 
 
+#ifndef RAPTOR_DISABLE_DEPRECATED
 /**
  * raptor_ntriples_term_as_string:
  * @term: N-Triples term.
  * 
- * @Deprecated: an internal debug function, do not use.
- *
  * Get a label for a #raptor_ntriples_term_type.
  * 
+ * @deprecated: an internal debug function, do not use.
+ *
  * Return value: a pointer to a constant string.
  **/
 const char *
@@ -135,7 +136,7 @@ raptor_ntriples_term_as_string(raptor_ntriples_term_type term)
 {
   return term_type_strings[(int)term];
 }
-
+#endif
 
 
 static void
@@ -161,15 +162,23 @@ raptor_ntriples_generate_statement(raptor_parser* parser,
     statement->subject=subject;
     statement->subject_type=RAPTOR_IDENTIFIER_TYPE_ANONYMOUS;
   } else {
-    subject_uri=raptor_new_uri(subject);
+    subject_uri=raptor_new_uri_v2(parser->world, subject);
+    if(!subject_uri) {
+      raptor_parser_error(parser, "Could not create subject uri '%s', skipping", subject);
+      goto cleanup;
+    }
     statement->subject=subject_uri;
     statement->subject_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
   }
 
- if(object_literal_datatype) {
-   datatype_uri=raptor_new_uri(object_literal_datatype);
-   object_literal_language=NULL;
- }
+  if(object_literal_datatype) {
+    datatype_uri=raptor_new_uri_v2(parser->world, object_literal_datatype);
+    if(!datatype_uri) {
+      raptor_parser_error(parser, "Could not create object literal datatype uri '%s', skipping", object_literal_datatype);
+      goto cleanup;
+    }
+    object_literal_language=NULL;
+  }
 
   /* Predicates in N-Triples are URIs but check for bad ordinals */
   if(!strncmp((const char*)predicate, "http://www.w3.org/1999/02/22-rdf-syntax-ns#_", 44)) {
@@ -178,7 +187,11 @@ raptor_ntriples_generate_statement(raptor_parser* parser,
       raptor_parser_error(parser, "Illegal ordinal value %d in property '%s'.", predicate_ordinal, predicate);
   }
   
-  predicate_uri=raptor_new_uri(predicate);
+  predicate_uri=raptor_new_uri_v2(parser->world, predicate);
+  if(!predicate_uri) {
+    raptor_parser_error(parser, "Could not create predicate uri '%s', skipping", predicate);
+    goto cleanup;
+  }
   statement->predicate_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
   statement->predicate=predicate_uri;
   
@@ -186,7 +199,11 @@ raptor_ntriples_generate_statement(raptor_parser* parser,
   statement->object_literal_language=NULL;
   statement->object_literal_datatype=NULL;
   if(object_type == RAPTOR_NTRIPLES_TERM_TYPE_URI_REF) {
-    object_uri=raptor_new_uri((const unsigned char*)object);
+    object_uri=raptor_new_uri_v2(parser->world, (const unsigned char*)object);
+    if(!object_uri) {
+      raptor_parser_error(parser, "Could not create object uri '%s', skipping", (const char *)object);
+      goto cleanup;
+    }
     statement->object=object_uri;
     statement->object_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
   } else if(object_type == RAPTOR_NTRIPLES_TERM_TYPE_BLANK_NODE) {
@@ -200,19 +217,20 @@ raptor_ntriples_generate_statement(raptor_parser* parser,
   }
 
   if(!parser->statement_handler)
-    return;
+    goto cleanup;
 
   /* Generate the statement; or is it fact? */
   (*parser->statement_handler)(parser->user_data, statement);
 
+  cleanup:
   if(subject_uri)
-    raptor_free_uri(subject_uri);
+    raptor_free_uri_v2(parser->world, subject_uri);
   if(predicate_uri)
-    raptor_free_uri(predicate_uri);
+    raptor_free_uri_v2(parser->world, predicate_uri);
   if(object_uri)
-    raptor_free_uri(object_uri);
+    raptor_free_uri_v2(parser->world, object_uri);
   if(datatype_uri)
-    raptor_free_uri(datatype_uri);
+    raptor_free_uri_v2(parser->world, datatype_uri);
 }
 
 
@@ -468,6 +486,7 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
 }
 
 
+#ifndef RAPTOR_DISABLE_DEPRECATED
 /**
  * raptor_ntriples_string_as_utf8_string:
  * @rdf_parser: parser object
@@ -477,7 +496,7 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
  * 
  * Turn an N-Triples string with escapes into a UTF-8 string.
  *
- * @Deprecated: This requires use of parser internals and was never in the public API header.
+ * @deprecated: This requires use of parser internals and was never in the public API header.
  * 
  * Return value: a new UTF-8 string
  **/
@@ -488,9 +507,14 @@ raptor_ntriples_string_as_utf8_string(raptor_parser* rdf_parser,
 {
   const unsigned char *start=src;
   size_t length=len;
-  unsigned char *dest=(unsigned char*)RAPTOR_MALLOC(cstring, len+1);
+  unsigned char *dest;
+  int rc;
+  
+  dest=(unsigned char*)RAPTOR_MALLOC(cstring, len+1);
+  if(!dest)
+    return NULL;
 
-  int rc=raptor_ntriples_term(rdf_parser, &start, dest, &length, dest_lenp,
+  rc=raptor_ntriples_term(rdf_parser, &start, dest, &length, dest_lenp,
                               '\0', RAPTOR_TERM_CLASS_FULL, 1);
   if(rc) {
     RAPTOR_FREE(cstring, dest);
@@ -498,7 +522,7 @@ raptor_ntriples_string_as_utf8_string(raptor_parser* rdf_parser,
   }
   return dest;
 }
-
+#endif
 
 
 static int
@@ -1136,8 +1160,8 @@ raptor_ntriples_parser_register_factory(raptor_parser_factory *factory)
 
 
 int
-raptor_init_parser_ntriples(void)
+raptor_init_parser_ntriples(raptor_world* world)
 {
-  return !raptor_parser_register_factory("ntriples",  "N-Triples",
+  return !raptor_parser_register_factory(world, "ntriples",  "N-Triples",
                                          &raptor_ntriples_parser_register_factory);
 }
